==> core-src.0.9.jar: shohaku/core/helpers/UnicodeEscapesCoder.java <==
/*
 * shohaku
 * Copyright (C) 2006  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.helpers;

/**
 * Unicode escapes の文字表現へのエンコードとデコード処理を行います。
 */
class UnicodeEscapesCoder {

    /**
     * Unicode escapes の文字表現を文字列へ変換して返却します。
     * 
     * @param src
     *            変換基の文字列
     * @return 変換後の文字列
     */
    static String decode(CharSequence src) {
        char aChar;
        int len = src.length();
        StringBuffer outBuffer = new StringBuffer(len);

        for (int x = 0; x < len;) {
            aChar = src.charAt(x++);
            if (aChar == '\\') {
                aChar = src.charAt(x++);
                if (aChar == 'u') {
                    // Read the xxxx
                    int value = 0;
                    for (int i = 0; i < 4; i++) {
                        aChar = src.charAt(x++);
                        switch (aChar) {
                        case '0':
                        case '1':
                        case '2':
                        case '3':
                        case '4':
                        case '5':
                        case '6':
                        case '7':
                        case '8':
                        case '9':
                            value = (value << 4) + aChar - '0';
                            break;
                        case 'a':
                        case 'b':
                        case 'c':
                        case 'd':
                        case 'e':
                        case 'f':
                            value = (value << 4) + 10 + aChar - 'a';
                            break;
                        case 'A':
                        case 'B':
                        case 'C':
                        case 'D':
                        case 'E':
                        case 'F':
                            value = (value << 4) + 10 + aChar - 'A';
                            break;
                        default:
                            throw new IllegalArgumentException("Malformed \\uxxxx encoding.");
                        }
                    }
                    outBuffer.append((char) value);
                } else {
                    switch (aChar) {
                    case 'b':
                        aChar = '\b';
                        break;
                    case 't':
                        aChar = '\t';
                        break;
                    case 'n':
                        aChar = '\n';
                        break;
                    case 'f':
                        aChar = '\f';
                        break;
                    case 'r':
                        aChar = '\r';
                        break;
                    case '"':
                    case '\'':
                    case '\\':
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                        break;
                    default:
                        throw new IllegalArgumentException("Malformed \\x encoding.");
                    }
                    outBuffer.append(aChar);
                }
            } else {
                outBuffer.append(aChar);
            }
        }
        return outBuffer.toString();
    }

    /**
     * 文字列を Unicode escapes の文字表現に変換して返却します。
     * 
     * @param src
     *            変換基の文字列
     * @return 変換後の文字列
     */
    static String encode(CharSequence src) {
        int len = src.length();
        int bufLen = len * 2;
        if (bufLen < 0) {
            bufLen = Integer.MAX_VALUE;
        }
        StringBuffer outBuffer = new StringBuffer(bufLen);

        for (int x = 0; x < len; x++) {
            char aChar = src.charAt(x);
            // unicode escape sequences
            outBuffer.append('\\');
            outBuffer.append('u');
            outBuffer.append(toHex((aChar >> 12) & 0xF));
            outBuffer.append(toHex((aChar >> 8) & 0xF));
            outBuffer.append(toHex((aChar >> 4) & 0xF));
            outBuffer.append(toHex(aChar & 0xF));
        }
        return outBuffer.toString();
    }

    /* 八進数の文字を返します。 */
    static char toHex(int nibble) {
        return hexDigit[(nibble & 0xF)];
    }

    /* 八進数数値のデータテーブル。 */
    static final char[] hexDigit = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };

}

==> core-src.0.91.jar: shohaku/core/helpers/UnicodeEscapesCoder.java <==
/*
 * shohaku
 * Copyright (C) 2006  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.helpers;

/**
 * Unicode escapes の文字表現へのエンコードとデコード処理を行います。
 */
class UnicodeEscapesCoder {

    /**
     * Unicode escapes の文字表現を文字列へ変換して返却します。
     * 
     * @param src
     *            変換基の文字列
     * @return 変換後の文字列
     */
    static String decode(CharSequence src) {
        final int len = src.length();
        final StringBuffer outBuffer = new StringBuffer(len);
        char aChar;
        for (int x = 0; x < len;) {
            aChar = src.charAt(x++);
            if (aChar == '\\') {
                aChar = src.charAt(x++);
                if (aChar == 'u') {
                    // Read the xxxx
                    int value = 0;
                    for (int i = 0; i < 4; i++) {
                        aChar = src.charAt(x++);
                        switch (aChar) {
                        case '0':
                        case '1':
                        case '2':
                        case '3':
                        case '4':
                        case '5':
                        case '6':
                        case '7':
                        case '8':
                        case '9':
                            value = (value << 4) + aChar - '0';
                            break;
                        case 'a':
                        case 'b':
                        case 'c':
                        case 'd':
                        case 'e':
                        case 'f':
                            value = (value << 4) + 10 + aChar - 'a';
                            break;
                        case 'A':
                        case 'B':
                        case 'C':
                        case 'D':
                        case 'E':
                        case 'F':
                            value = (value << 4) + 10 + aChar - 'A';
                            break;
                        default:
                            throw new IllegalArgumentException("Malformed \\uxxxx encoding.");
                        }
                    }
                    outBuffer.append((char) value);
                } else {
                    switch (aChar) {
                    case 'b':
                        aChar = '\b';
                        break;
                    case 't':
                        aChar = '\t';
                        break;
                    case 'n':
                        aChar = '\n';
                        break;
                    case 'f':
                        aChar = '\f';
                        break;
                    case 'r':
                        aChar = '\r';
                        break;
                    case '"':
                    case '\'':
                    case '\\':
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                        break;
                    default:
                        throw new IllegalArgumentException("Malformed \\x encoding.");
                    }
                    outBuffer.append(aChar);
                }
            } else {
                outBuffer.append(aChar);
            }
        }
        return outBuffer.toString();
    }

    /**
     * 文字列を Unicode escapes の文字表現に変換して返却します。
     * 
     * @param src
     *            変換基の文字列
     * @return 変換後の文字列
     */
    static String encode(CharSequence src) {
        final int len = src.length();
        int bufLen = len * 2;
        if (bufLen < 0) {
            bufLen = Integer.MAX_VALUE;
        }
        final StringBuffer outBuffer = new StringBuffer(bufLen);
        for (int x = 0; x < len; x++) {
            char aChar = src.charAt(x);
            // unicode escape sequences
            outBuffer.append('\\');
            outBuffer.append('u');
            outBuffer.append(toHex((aChar >> 12) & 0xF));
            outBuffer.append(toHex((aChar >> 8) & 0xF));
            outBuffer.append(toHex((aChar >> 4) & 0xF));
            outBuffer.append(toHex(aChar & 0xF));
        }
        return outBuffer.toString();
    }

    /* 八進数の文字を返します。 */
    static char toHex(int nibble) {
        return hexDigit[(nibble & 0xF)];
    }

    /* 八進数数値のデータテーブル。 */
    static final char[] hexDigit = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };

}
